//
//  MasterViewController.swift
//  SwiftCollectionView
//
//  Created by Merlyn on 11/24/14.
//  Copyright (c) 2014 The App Lady. All rights reserved.
//

import UIKit

let reuseIdentifier = "Cell"

class MasterViewController: UIViewController, UISearchBarDelegate, UIGestureRecognizerDelegate {

    @IBOutlet weak var editButton: UIBarButtonItem!
    @IBOutlet weak var searchBar: UISearchBar!
    @IBOutlet weak var collectionView: UICollectionView!
    var icon: PhotoCell!
    var imageFileNames = [String]()
    var selectedPhotoName = String()
    var isSearchOn = false
    var searchResults = [String]()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Set the collection view's backgroundcolor to display an image
        self.collectionView!.backgroundColor = UIColor(patternImage: UIImage(named: "purty_wood")!)
        
        // Register the  class with the collection view
        self.collectionView!.registerClass(PhotoCell.self, forCellWithReuseIdentifier: reuseIdentifier)
        
        // Add objects in the collection view dataSource, imageFileNames Array
        loadImages()
        
        //1. Create a tap gesture recognizer object
        let tapRecognizer = UITapGestureRecognizer()
        tapRecognizer.numberOfTapsRequired = 2
        
        //2. Set the target function for the tap gesture recognizer object
        tapRecognizer.addTarget(self, action: "collectionViewBackgroundTapped")
        
        //3. Add the tap gesture recognizer to the collection view
        self.collectionView!.addGestureRecognizer(tapRecognizer)
    }

    // Get images from the app bundle and put them in the cv dataSource object
    func loadImages() {
        // Get reference to the app's main bundle
        var mainBundle: NSBundle = NSBundle.mainBundle()
        
        // Fetch all jpg files from the main bundle
        var results:NSArray = mainBundle.pathsForResourcesOfType("jpg",inDirectory:nil)
        //println(results)
        
        // Convert NSArray to Swift Array
        var dirPaths: Array = results as AnyObject as! [String]
        //println(dirPaths)
        for path in dirPaths {
            // Convert the String values stored in path to an array
            var objectArray = path.componentsSeparatedByString("/")
            
            // Remove last the object from the objectArray and put it the imageFileNames variable
            imageFileNames.append(objectArray.removeLast())
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "photoDetail" {
            // Put the destination view controller in a variable
            let controller = (segue.destinationViewController as! UINavigationController).topViewController as! PhotoDetailViewController
            
            // Pass the selectedPhoto to the destination view controller
            controller.selectedPhotoName = selectedPhotoName
        }
    }

    @IBAction func unwindSegue(segue: UIStoryboardSegue) {
        if segue.identifier == "showPhotos" {
        }
    }
    
    // MARK: UICollectionViewDataSource
    
    func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        // Return the number of sections
        return 1
    }
    
    func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        // Return the number of items in the section
        // Return the number of items in a section (number of photos in an album)
        if isSearchOn == true && !searchResults.isEmpty {
            return searchResults.count
        } else {
            return imageFileNames.count
        }
    }
    
    func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        // Initialize the reusable Collection View Cell with our custom class
        icon = collectionView.dequeueReusableCellWithReuseIdentifier(reuseIdentifier, forIndexPath: indexPath) as! PhotoCell
        var photoName = String()
        
        // Initialize the photoName variable with an item in the searchResults array or an item in the imageFileNames array
        if isSearchOn == true && !searchResults.isEmpty {
            photoName = searchResults[indexPath.item]
        } else {
            photoName = imageFileNames[indexPath.item]
        }

        // Configure the collection view cell
        icon.imageView.image = UIImage(named: photoName)
        var stringArray: Array = photoName.componentsSeparatedByString(".")
        icon.caption.text = stringArray[0]
        
        // Return the cell
        return icon
    }

    // MARK: UICollectionViewDelegate
    
    func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        // Put the selected collection view cell's photo file name in a variable
        if isSearchOn == true && !searchResults.isEmpty {
            selectedPhotoName = searchResults[indexPath.row] as String
        } else {
            selectedPhotoName = imageFileNames[indexPath.row] as String
        }
        
        // Pass control to the PhotoDetailViewController
        self.performSegueWithIdentifier("photoDetail", sender:self)
    }

    // MARK: UISearchBarDelegate functions
    
    // This function is fired when the user tap the Search Bar's Cancel button
    func searchBarCancelButtonClicked(searchBar: UISearchBar) {
        searchBar.text = nil // Clear out the Search Bar's text field
        searchBar.showsCancelButton = false // Hide the Search Bar's Cancel button
        searchBar.resignFirstResponder()  // Dismiss the keyboard
        isSearchOn = false // Turn off search function
        self.collectionView.reloadData()  // Refresh the collection view
    }
    
    // this function is fired when the user start entering text in the Search Bar's text field
    func searchBar(searchBar: UISearchBar, textDidChange searchText: String) {
        searchBar.showsCancelButton = true  // Show the Search Bar's Cancel button
        if !searchText.isEmpty {
            isSearchOn = true  // Turn on searching function
            self.filterContentForSearchText() // Search the collection view's dataSource
            self.collectionView.reloadData()
        }
    }
    
    // MARK: Helper functions for the UISearchBarDelegate functions
    
    // This function search the collection view's dataSource object (imageFileNames)for an item that match
    // text entered in the Search Bar's text field. If any items found, add them in the searchResults array
    func filterContentForSearchText() {
        // Remove all elements from the searchResults array
        searchResults.removeAll(keepCapacity: false)
        
        // Loop throught the collection view's dataSource object
        for imageFileName in imageFileNames {
            let stringToLookFor = imageFileName as NSString
            let sourceString = searchBar.text as NSString
            
            if stringToLookFor.localizedCaseInsensitiveContainsString(sourceString as String) {
                // Match found, so add it to the searchResults array variable
                searchResults.append(imageFileName)
            }
        }
    }
    
    // Target function for the tap gesture recognizer object created in the viewDidLoad function
    func collectionViewBackgroundTapped() {
        // Dismiss the keyboard that's shown on the device's screen
        searchBar.resignFirstResponder()
    }
    
    /***********************
    This function allow the default tap gesture added to the collection view cell,
    an the collection view's background to to work simultaneously
    **************************/
    func gestureRecognizer(gestureRecognizer: UIGestureRecognizer, shouldRequireFailureOfGestureRecognizer otherGestureRecognizer: UIGestureRecognizer) -> Bool {
        // Perform default tap gesture
        return true
    }

}
